==> patientflow-0.4.2.tar.gz: patientflow-0.4.2/README.md <==
# PatientFlow: Predicting demand for hospital beds using real-time data

[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/pre-commit/pre-commit)
[![Tests status][tests-badge]][tests-link]
[![Linting status][linting-badge]][linting-link]
[![Documentation status][documentation-badge]][documentation-link]
[![License][license-badge]](./LICENSE.md)

<!--
[![PyPI version][pypi-version]][pypi-link]
[![Conda-Forge][conda-badge]][conda-link]
[![PyPI platforms][pypi-platforms]][pypi-link]
-->

<!-- prettier-ignore-start -->
[tests-badge]:              https://github.com/zmek/patientflow/actions/workflows/tests.yml/badge.svg
[tests-link]:               https://github.com/zmek/patientflow/actions/workflows/tests.yml
[linting-badge]:            https://github.com/zmek/patientflow/actions/workflows/linting.yml/badge.svg
[linting-link]:             https://github.com/zmek/patientflow/actions/workflows/linting.yml
[documentation-badge]:      https://github.com/zmek/patientflow/actions/workflows/docs.yml/badge.svg
[documentation-link]:       https://github.com/zmek/patientflow/actions/workflows/docs.yml
[conda-badge]:              https://img.shields.io/conda/vn/conda-forge/patientflow
[conda-link]:               https://github.com/conda-forge/patientflow-feedstock
[license-badge]:            https://img.shields.io/badge/License-MIT-yellow.svg
[![ORCID](https://img.shields.io/badge/ORCID-0000--0001--7389--1527-green.svg)](https://orcid.org/0000-0001-7389-1527)

<!-- [pypi-link]:                https://pypi.org/project/patientflow/
[pypi-platforms]:           https://img.shields.io/pypi/pyversions/patientflow
[pypi-version]:             https://img.shields.io/pypi/v/patientflow -->
<!-- prettier-ignore-end -->

## Summary

`patientflow`, a Python package, converts patient-level predictions into output that is useful for bed managers in hospitals.

We originally developed this code for University College London Hospitals (UCLH) to predict the number of emergency admissions they should expect within the next eight hours. Our method used real-time data from their Electronic Health Record (EHR) system. We wrote code to convert patient-level data, extracted from the EHR at a point in time, into predicted numbers of admissions in the following 4 or 8 hours. We also wrote code to help us evaluate the predictions.

We have created the `patientflow` python package to make it convenient for others to adopt our approach. Its purpose is to predict bed demand for groups of hospital patients at a point in time. The package is organised around the following concepts:

- Prediction time: A moment in the day at which predictions are to be made, for example 09:30.
- Patient snapshot: A summary of data from the EHR capturing is known about a single patient at the prediction time. Each patient snapshot has a date and a prediction time associated with it.
- Group snapshot: A set of patients snapshots. Each group snapshot has a date and a prediction time associated with it.
- Prediction window: A period of hours that begins at the prediction time.

The modelling functions in `patientflow` are designed to receive a group snapshot as an input, and to predict something about that group's demand for beds between the prediction moment and the end of the prediction window. For example, that group could be the patients currently in the Emergency Department (ED), and the predictions could be the number of beds needed by those patients in the next 8 hours. The output is a probability distribution over the number of beds needed. The package includes functions to generate predictions at both patient and group level, to visualise predicted probability distributions, and to evaluate them.

This snapshot-based approach to predicting demand generalises to other aspects of patient flow in hospitals, such as predictions of how many patients from a clinical specialty will be discharged. A series of notebooks demonstrates the use of the package. We show how to prepare your data and train models based on a snapshot approach. The repository includes a synthetic dataset, and an anonymised patient dataset, based on real data from UCLH is available on [Zenodo](https://zenodo.org/records/14866057). Both the synthetic and the real dataset have been prepared in a snapshot structure.

## What `patientflow` is for:

- Predicting patient flow in hospitals: The package can be used by researchers or analysts who want to predict numbers of emergency admissions, discharges or transfers between units.
- Short-term operational planning: The predictions produced by this package are designed for bed managers who need to make decisions within an 4-16 hour timeframe.
- Working with real-time data: The design assumes that data from an electronic health record (EHR) is available in real-time, or near to real-time.
- Point-in-time analysis: The package works by taking snapshots of groups of patients who are in the hospital at a particular moment, and making predictions about whether a non-clinical outcome like admission or discharge will occur with a short time horizon.

## What `patientflow` is NOT for:

- Long-term capacity planning: The package focuses on immediate operational demand (hours ahead), not strategic planning over weeks or months.
- Making decisions about individual patients: The package relies on data entered into the EHR by clinical staff looking after patients, but the patient-level predictions it generates should not be used to influence their decision-making.
- General hospital analytics: The package is designed for short-term bed management, not broader hospital analytics like long-term demand and capacity planning.
- Predicting what happens _after_ a hospital visit has finished: While historical data might train underlying models, the package itself focuses on patients currently in the hospital or soon to arrive.
- Replacing human judgment: The predictions are meant to augment the information available to bed managers, but not to automate bed management decisions.

## This package will help you if you want to:

- Make predictions for unfinished patient visits: The package is designed for making predictions when outcomes at the end of the visit are as yet unknown, and evaluating those predictions against what actually happened.
- Convert individual patient predictions to group-level insights: As bed numbers are the currency used by bed managers, the package generates bed count distributions; you may find this kind of output will help you interest hospital site and operations managers in your predictions.
- Develop your own predictive models of emergency demand: The repository includes a fully worked example of how to convert historical data from Emergency Department visits into snapshots, and use the snapshots to train models that predict numbers of emergency beds.

## This package will NOT help you if:

- You work with time series data: `patientflow` works with snapshots of a hospital visit summarising what is in the patient record up to that point in time. It would need modification to accept time series data formats.
- You want to predict clinical outcomes: the approach is designed for the management of hospital sites, not the management of patient care.

## Mathematical assumptions underlying the conversion from individual to group predictions:

- Independence of patient requirements: The package assumes that individual patient requirements (eg for admission) are conditionally independent.
- Bernoulli outcome model: Each patient outcome is modelled as a Bernoulli trial with its own probability, and the package computes a probability distribution for the sum of these independent trials.
- Different levels of aggregation: The package can calculate probability distributions for compound scenarios (such as the probability of a patient being admitted, assigned to a specific specialty if admitted, and being admitted within the prediction window) and for patient subgroups (like distributions by age or gender). In all cases, the independence assumption between patients is maintained.

## Getting started

- Exploration: Start with the [notebooks README](https://github.com/UCL-CORU/patientflow/blob/main/notebooks/README.md) to get an outline of what is included in the notebooks, and read the [patientflow README](https://github.com/UCL-CORU/patientflow/tree/main/src/patientflow#readme) for an overview of the Python package.
- Installation: Follow the instructions below to set up the environment and install necessary dependencies in your own environment.
- Configuration: Repurpose config.yaml to configure the package to your own data and user requirements.

### Prerequisites

`patientflow` requires Python 3.10.

### Installation

To install the latest development version, clone it first (so that you have access to the synthetic data and the notebooks) and then install it.

```sh
git clone https://github.com/zmek/patientflow.git
cd patientflow
pip install -e ".[test]" #this will install the code in test mode

```

### Development Installation (optional)

If you plan to contribute to the project or run documentation locally, install the development and documentation dependencies:

```sh
# For development tools (linting, formatting, etc.)
pip install -e ".[dev]"

# For building documentation
pip install -e ".[docs]"

# For both development and documentation
pip install -e ".[dev,docs]"
```

Navigate to the patientflow folder and run tests to confirm that the installation worked correctly. This command will only work from the root repository. (To date, this has only been tested on Linux and Mac OS machines. If you are running Windows, there may be errors we don't know about. Please raise an issue on Github in that case.)

```sh
pytest
```

If you get errors running the pytest command, there may be other installations needed on your local machine.

### Building and Viewing Documentation (optional)

After installing the documentation dependencies, you can build and view the documentation locally:

```sh
# Build and serve the documentation with live reloading
mkdocs serve

# Or just build the documentation
mkdocs build
```

The documentation will be available at http://127.0.0.1:8000/ when using `mkdocs serve`.

## Using the notebooks in this repository

The notebooks in this repository demonstrate the use of some of the functions provided in `patientflow`. The cell output shows the results of running the notebooks. If you want to run them yourself, you have two options

- step through the notebooks using the real patient datasets that were used to prepare them. For this you need to request access on [Zenodo](https://zenodo.org/records/14866057) to real patient data
- step through the notebooks using synthetic data. You will need to copy the two csv files from `data-synthetic`into your `data-public` folder or change the source in the each notebook. If you use synthetic data, you will not see the same cell output.

## About the UCLH implementation

This repository includes a set of notebooks (prefixed with 4) that show a fully worked example of the implementation of the patientflow package at University College London Hospitals (UCLH). As noted above, please request access to the UCLH dataset via Zenodo.

There is also a Python script that illustrates the training of the models that predict emergency demand at UCLH and saves them in your local environment using following commands (by default this will run with the synthetic data in its current location; change the `data_folder_name` parameter if you have downloaded the Zenodo dataset in `data-public`)

```sh
cd src
python -m patientflow.train.emergency_demand --data_folder_name=data-synthetic
```

The `data_folder_name`argument specifies the name of the folder containing data. The function expects this folder to be directly below the root of the repository.

## Contributing to PatientFlow

We welcome contributions to the patientflow project. To contribute, follow the instructions below.

### Development Workflow

1. Fork the repository on GitHub
2. Clone your fork locally and set up your development environment following the [installation instructions](#installation) above, making sure to install the development dependencies.
3. Create a new branch for your changes:
   ```sh
   git checkout -b feature/your-feature-name
   ```
4. Make your changes following the code style guidelines
5. Run tests as described in the installation section to ensure your changes don't break existing functionality
6. Update documentation if needed using the documentation tools mentioned above
7. Commit your changes with a descriptive message

### Code Style Guidelines

- Follow PEP 8 guidelines for Python code
- Use type hints where appropriate
- Write docstrings for all functions, classes, and modules
- Add unit tests for new functionality

### Submitting Your Contribution

1. Push your changes to your forked repository:
   ```sh
   git push origin feature/your-feature-name
   ```
2. Open a pull request from your fork to the main repository
   - Provide a clear title and description
   - Reference any relevant issues
   - Include screenshots if applicable
3. Address any feedback or review comments

### Reporting Issues

If you find a bug or have a suggestion:

1. Check existing issues to avoid duplicates
2. Open a new issue describing:
   - What you expected to happen
   - What actually happened
   - Steps to reproduce
   - Your environment details (OS, Python version, etc.)

Thank you for contributing!

## Roadmap

- [x] Initial Research
- [x] Minimum viable product <-- You are Here
- [ ] Alpha Release
- [ ] Feature-Complete Release

### Project Team

- [Dr Zella King](https://github.com/zmek), Clinical Operational Research Unit (CORU), University College London (UCL)([zella.king@ucl.ac.uk](mailto:zella.king@ucl.ac.uk))
- [Jon Gillham](https://github.com/jongillham), Institute of Health Informatics, UCL
- Professor Sonya Crowe, CORU
- Professor Martin Utley, CORU

## Acknowledgements

The [py-pi template](https://github.com/health-data-science-OR/pypi-template) developed by [Tom Monks](https://github.com/TomMonks) inspired us to create a Python package. This repository is based on a template developed by the [Centre for Advanced Research Computing](https://ucl.ac.uk/arc), University College London. We are grateful to [Lawrence Lai](https://github.com/lawrencelai) for creation of the synthetic dataset. MAPS QR Policy Funding from by University College London contributed to the construction of the repository.

The development of this repository/package was funded by UCL's QR Policy Support Fund, which is funded by [Research England](https://www.ukri.org/councils/research-england/).

==> patientflow-0.4.3.tar.gz: patientflow-0.4.3/README.md <==
# PatientFlow: Predicting demand for hospital beds using real-time data

[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/pre-commit/pre-commit)
[![Tests status][tests-badge]][tests-link]
[![Linting status][linting-badge]][linting-link]
[![Documentation status][documentation-badge]][documentation-link]
[![License][license-badge]](./LICENSE.md)
[![PyPI version][pypi-version]][pypi-link]
[![PyPI platforms][pypi-platforms]][pypi-link]

<!-- prettier-ignore-start -->
[tests-badge]:              https://github.com/zmek/patientflow/actions/workflows/tests.yml/badge.svg
[tests-link]:               https://github.com/zmek/patientflow/actions/workflows/tests.yml
[linting-badge]:            https://github.com/zmek/patientflow/actions/workflows/linting.yml/badge.svg
[linting-link]:             https://github.com/zmek/patientflow/actions/workflows/linting.yml
[documentation-badge]:      https://github.com/zmek/patientflow/actions/workflows/docs.yml/badge.svg
[documentation-link]:       https://github.com/zmek/patientflow/actions/workflows/docs.yml
[license-badge]:            https://img.shields.io/badge/License-MIT-yellow.svg
[pypi-link]:                https://pypi.org/project/patientflow/
[pypi-platforms]:           https://img.shields.io/pypi/pyversions/patientflow
[pypi-version]:             https://img.shields.io/pypi/v/patientflow
[![ORCID](https://img.shields.io/badge/ORCID-0000--0001--7389--1527-green.svg)](https://orcid.org/0000-0001-7389-1527)
<!-- prettier-ignore-end -->

## Summary

`patientflow`, a Python package, converts patient-level predictions into output that is useful for bed managers in hospitals.

We originally developed this code for University College London Hospitals (UCLH) to predict the number of emergency admissions they should expect within the next eight hours. Our method used real-time data from their Electronic Health Record (EHR) system. We wrote code to convert patient-level data, extracted from the EHR at a point in time, into predicted numbers of admissions in the following 4 or 8 hours. We also wrote code to help us evaluate the predictions.

We have created the `patientflow` python package to make it convenient for others to adopt our approach. Its purpose is to predict bed demand for groups of hospital patients at a point in time. The package is organised around the following concepts:

- Prediction time: A moment in the day at which predictions are to be made, for example 09:30.
- Patient snapshot: A summary of data from the EHR capturing is known about a single patient at the prediction time. Each patient snapshot has a date and a prediction time associated with it.
- Group snapshot: A set of patients snapshots. Each group snapshot has a date and a prediction time associated with it.
- Prediction window: A period of hours that begins at the prediction time.

The modelling functions in `patientflow` are designed to receive a group snapshot as an input, and to predict something about that group's demand for beds between the prediction moment and the end of the prediction window. For example, that group could be the patients currently in the Emergency Department (ED), and the predictions could be the number of beds needed by those patients in the next 8 hours. The output is a probability distribution over the number of beds needed. The package includes functions to generate predictions at both patient and group level, to visualise predicted probability distributions, and to evaluate them.

This snapshot-based approach to predicting demand generalises to other aspects of patient flow in hospitals, such as predictions of how many patients from a clinical specialty will be discharged. A series of notebooks demonstrates the use of the package. We show how to prepare your data and train models based on a snapshot approach. The repository includes a synthetic dataset, and an anonymised patient dataset, based on real data from UCLH is available on [Zenodo](https://zenodo.org/records/14866057). Both the synthetic and the real dataset have been prepared in a snapshot structure.

## What `patientflow` is for:

- Predicting patient flow in hospitals: The package can be used by researchers or analysts who want to predict numbers of emergency admissions, discharges or transfers between units.
- Short-term operational planning: The predictions produced by this package are designed for bed managers who need to make decisions within an 4-16 hour timeframe.
- Working with real-time data: The design assumes that data from an electronic health record (EHR) is available in real-time, or near to real-time.
- Point-in-time analysis: The package works by taking snapshots of groups of patients who are in the hospital at a particular moment, and making predictions about whether a non-clinical outcome like admission or discharge will occur with a short time horizon.

## What `patientflow` is NOT for:

- Long-term capacity planning: The package focuses on immediate operational demand (hours ahead), not strategic planning over weeks or months.
- Making decisions about individual patients: The package relies on data entered into the EHR by clinical staff looking after patients, but the patient-level predictions it generates should not be used to influence their decision-making.
- General hospital analytics: The package is designed for short-term bed management, not broader hospital analytics like long-term demand and capacity planning.
- Predicting what happens _after_ a hospital visit has finished: While historical data might train underlying models, the package itself focuses on patients currently in the hospital or soon to arrive.
- Replacing human judgment: The predictions are meant to augment the information available to bed managers, but not to automate bed management decisions.

## This package will help you if you want to:

- Make predictions for unfinished patient visits: The package is designed for making predictions when outcomes at the end of the visit are as yet unknown, and evaluating those predictions against what actually happened.
- Convert individual patient predictions to group-level insights: As bed numbers are the currency used by bed managers, the package generates bed count distributions; you may find this kind of output will help you interest hospital site and operations managers in your predictions.
- Develop your own predictive models of emergency demand: The repository includes a fully worked example of how to convert historical data from Emergency Department visits into snapshots, and use the snapshots to train models that predict numbers of emergency beds.

## This package will NOT help you if:

- You work with time series data: `patientflow` works with snapshots of a hospital visit summarising what is in the patient record up to that point in time. It would need modification to accept time series data formats.
- You want to predict clinical outcomes: the approach is designed for the management of hospital sites, not the management of patient care.

## Mathematical assumptions underlying the conversion from individual to group predictions:

- Independence of patient requirements: The package assumes that individual patient requirements (eg for admission) are conditionally independent.
- Bernoulli outcome model: Each patient outcome is modelled as a Bernoulli trial with its own probability, and the package computes a probability distribution for the sum of these independent trials.
- Different levels of aggregation: The package can calculate probability distributions for compound scenarios (such as the probability of a patient being admitted, assigned to a specific specialty if admitted, and being admitted within the prediction window) and for patient subgroups (like distributions by age or gender). In all cases, the independence assumption between patients is maintained.

## Getting started

- Exploration: Start with the [notebooks README](https://github.com/UCL-CORU/patientflow/blob/main/notebooks/README.md) to get an outline of what is included in the notebooks, and read the [patientflow README](https://github.com/UCL-CORU/patientflow/tree/main/src/patientflow#readme) for an overview of the Python package.
- Installation: Follow the instructions below to set up the environment and install necessary dependencies in your own environment.
- Configuration: Repurpose config.yaml to configure the package to your own data and user requirements.

### Prerequisites

`patientflow` requires Python 3.10.

### Installation

You can install `patientflow` directly from PyPI:

```sh
pip install patientflow
```

To access the example notebooks and synthetic data, clone the repository:

```sh
git clone https://github.com/ucl-coru/patientflow.git
cd patientflow
```

### Development Installation (optional)

If you want to contribute or modify the code, or run documentation locally, install the development and documentation dependencies:

```sh
# For contributors (includes development tools, documentation, and testing)
pip install -e ".[dev,docs,test]"

# For specific purposes only:
# For development tools (linting, formatting, etc.)
pip install -e ".[dev]"

# For building documentation
pip install -e ".[docs]"

# For running tests
pip install -e ".[test]"
```

Navigate to the patientflow folder and run tests to confirm that the installation worked correctly. This command will only work from the root repository. (To date, this has only been tested on Linux and Mac OS machines. If you are running Windows, there may be errors we don't know about. Please raise an issue on Github in that case.)

```sh
pytest
```

If you get errors running the pytest command, there may be other installations needed on your local machine.

### Building and Viewing Documentation (optional)

After installing the documentation dependencies, you can build and view the documentation locally:

```sh
# Build and serve the documentation with live reloading
mkdocs serve

# Or just build the documentation
mkdocs build
```

The documentation will be available at http://127.0.0.1:8000/ when using `mkdocs serve`.

## Using the notebooks in this repository

The notebooks in this repository demonstrate the use of some of the functions provided in `patientflow`. The cell output shows the results of running the notebooks. If you want to run them yourself, you have two options

- step through the notebooks using the real patient datasets that were used to prepare them. For this you need to request access on [Zenodo](https://zenodo.org/records/14866057) to real patient data
- step through the notebooks using synthetic data. You will need to copy the two csv files from `data-synthetic`into your `data-public` folder or change the source in the each notebook. If you use synthetic data, you will not see the same cell output.

## About the UCLH implementation

This repository includes a set of notebooks (prefixed with 4) that show a fully worked example of the implementation of the patientflow package at University College London Hospitals (UCLH). As noted above, please request access to the UCLH dataset via Zenodo.

There is also a Python script that illustrates the training of the models that predict emergency demand at UCLH and saves them in your local environment using following commands (by default this will run with the synthetic data in its current location; change the `data_folder_name` parameter if you have downloaded the Zenodo dataset in `data-public`)

```sh
cd src
python -m patientflow.train.emergency_demand --data_folder_name=data-synthetic
```

The `data_folder_name`argument specifies the name of the folder containing data. The function expects this folder to be directly below the root of the repository.

## Contributing to PatientFlow

We welcome contributions to the patientflow project. To contribute, follow the instructions below.

### Development Workflow

1. Fork the repository on GitHub
2. Clone your fork locally and set up your development environment following the [installation instructions](#installation) above, making sure to install the development dependencies.
3. Create a new branch for your changes:
   ```sh
   git checkout -b feature/your-feature-name
   ```
4. Make your changes following the code style guidelines
5. Run tests as described in the installation section to ensure your changes don't break existing functionality
6. Update documentation if needed using the documentation tools mentioned above
7. Commit your changes with a descriptive message

### Code Style Guidelines

- Follow PEP 8 guidelines for Python code
- Use type hints where appropriate
- Write docstrings for all functions, classes, and modules
- Add unit tests for new functionality

### Submitting Your Contribution

1. Push your changes to your forked repository:
   ```sh
   git push origin feature/your-feature-name
   ```
2. Open a pull request from your fork to the main repository
   - Provide a clear title and description
   - Reference any relevant issues
   - Include screenshots if applicable
3. Address any feedback or review comments

### Reporting Issues

If you find a bug or have a suggestion:

1. Check existing issues to avoid duplicates
2. Open a new issue describing:
   - What you expected to happen
   - What actually happened
   - Steps to reproduce
   - Your environment details (OS, Python version, etc.)

Thank you for contributing!

## Roadmap

- [x] Initial Research
- [x] Minimum viable product
- [x] Alpha Release (PyPI Package) <-- You are Here
- [ ] Feature-Complete Release

### Project Team

- [Dr Zella King](https://github.com/zmek), Clinical Operational Research Unit (CORU), University College London (UCL)([zella.king@ucl.ac.uk](mailto:zella.king@ucl.ac.uk))
- [Jon Gillham](https://github.com/jongillham), Institute of Health Informatics, UCL
- Professor Sonya Crowe, CORU
- Professor Martin Utley, CORU

## Acknowledgements

The [py-pi template](https://github.com/health-data-science-OR/pypi-template) developed by [Tom Monks](https://github.com/TomMonks) inspired us to create a Python package. This repository is based on a template developed by the [Centre for Advanced Research Computing](https://ucl.ac.uk/arc), University College London. We are grateful to [Lawrence Lai](https://github.com/lawrencelai) for creation of the synthetic dataset. MAPS QR Policy Funding from by University College London contributed to the construction of the repository.

The development of this repository/package was funded by UCL's QR Policy Support Fund, which is funded by [Research England](https://www.ukri.org/councils/research-england/).
